module PidLock (withLockedPidFile) where

import Control.Exception (bracket)
import System.Directory (removeFile)
import System.IO (SeekMode(AbsoluteSeek))
import System.Posix.IO (LockRequest(WriteLock),OpenMode(WriteOnly),closeFd,defaultFileFlags,fdWrite,openFd,setLock)
import System.Posix.Files (setFdSize)
import System.Posix.Process (getProcessID)

withLockedPidFile :: FilePath -> IO c -> IO c
withLockedPidFile pidFile op = bracket
  (do fd <- openFd pidFile WriteOnly (Just 0o666) defaultFileFlags
      setLock fd (WriteLock,AbsoluteSeek,0,0)
      setFdSize fd 0
      _ <- getProcessID >>= fdWrite fd . (++"\n") . show
      return fd)
  (\fd -> closeFd fd >> removeFile pidFile)
  (const op)
